/**************************************************************** 
 * Description: P1084
 代码解释：
1、树的构建：通过邻接表构建树，每条边都被记录两次（因为是无向边），并通过add函数添加边。
2、树的预处理：使用BFS从根节点出发，计算每个节点的深度和倍增数组f[][]，倍增数组用于快速查找节点的祖先。
3、路径覆盖的检查：使用dfs函数从根节点开始检查每条路径是否都能被军队覆盖。通过check函数使用贪心策略来检查在当前时间限制下，是否能覆盖所有路径。
4、二分搜索最小时间：利用二分搜索，找出能覆盖所有路径的最小时间。
5、输出：最终输出能覆盖所有路径所需的最小时间。如果无法覆盖，则输出-1。
 * Author: Alex Li
 * Date: 2024-08-16 14:06:36
 * LastEditTime: 2024-08-16 14:07:59
****************************************************************/

#include<iostream>
#include<cstdio>
#include<cstring>
#include<queue>
#include<cmath>
#include<algorithm>
#define ll long long
using namespace std;

const int N = 6e4;  // 定义常量N，表示最大城市数

int n, m, t, tot = 0, atot = 0, btot = 0, ctot = 0;
int d[N], query[N], f[N][20];  // d[]存储节点的深度，query[]存储军队位置，f[][]存储节点的祖先
int ver[2*N], edge[2*N], Next[2*N], head[N];  // 用于存储树的邻接表
bool ok, sta[N], need[N];
ll ans, tim[N], ned[N], dist[N][20];  // tim[]和ned[]用于贪心匹配

pair<ll, int> h[N];  // 用于存储距离剩余时间和节点的对
queue<int> q;  // 用于BFS

void add(int x, int y, int z) {
    ver[++tot] = y, edge[tot] = z, Next[tot] = head[x], head[x] = tot;
    ver[++tot] = x, edge[tot] = z, Next[tot] = head[y], head[y] = tot;
}
// 添加边（无向图存储）

void bfs() {
    q.push(1);  // 从根节点开始BFS
    d[1] = 1;  // 根节点深度为1

    while (!q.empty()) {
        int x = q.front(); q.pop();
        for (int i = head[x]; i; i = Next[i]) {
            int y = ver[i];
            if (d[y]) continue;  // 如果已经访问过，则跳过

            d[y] = d[x] + 1;  // 更新节点y的深度
            f[y][0] = x;  // y的第一层祖先是x
            dist[y][0] = edge[i];  // y到其父节点的距离

            for (int j = 1; j <= t; j++) {
                f[y][j] = f[f[y][j-1]][j-1];
                dist[y][j] = dist[y][j-1] + dist[f[y][j-1]][j-1];
            }
            q.push(y);
        }
    }
}
// 树的BFS遍历，预处理倍增数组f[][]和dist[][]

bool dfs(int x) {
    bool hasChild = false;
    if (sta[x]) return true;  // 如果节点x已驻扎军队，直接返回true

    for (int i = head[x]; i; i = Next[i]) {
        int y = ver[i];
        if (d[y] < d[x]) continue;  // 避免访问父节点

        hasChild = true;
        if (!dfs(y)) return false;  // 如果某个子节点无法覆盖，返回false
    }

    if (!hasChild) return false;  // 如果是叶子节点且未被覆盖，返回false

    return true;
}
// 深度优先搜索，检查从节点x到其所有叶子节点的路径是否都已覆盖

bool check(ll lim) {
    memset(sta, 0, sizeof(sta));
    memset(tim, 0, sizeof(tim));
    memset(ned, 0, sizeof(ned));
    memset(h, 0, sizeof(h));
    memset(need, 0, sizeof(need));

    atot = 0, btot = 0, ctot = 0;  // 初始化计数器

    for (int i = 1; i <= m; i++) {
        ll x = query[i], cnt = 0;
        for (int j = t; j >= 0; j--) {
            if (f[x][j] > 1 && cnt + dist[x][j] <= lim) {
                cnt += dist[x][j];
                x = f[x][j];
            }
        }
        if (f[x][0] == 1 && cnt + dist[x][0] <= lim) {
            h[++ctot] = make_pair(lim - cnt - dist[x][0], x);
        } else {
            sta[x] = true;
        }
    }
    // 将每支军队尽可能上移，并记录空闲军队的位置和剩余距离

    for (int i = head[1]; i; i = Next[i]) {
        if (!dfs(ver[i])) need[ver[i]] = true;
    }
    // 通过DFS寻找所有需要驻扎的路径节点

    sort(h + 1, h + ctot + 1);  // 对空闲军队按剩余时间排序

    for (int i = 1; i <= ctot; i++) {
        if (need[h[i].second] && h[i].first < dist[h[i].second][0]) {
            need[h[i].second] = false;
        } else {
            tim[++atot] = h[i].first;
        }
    }
    // 尽可能匹配空闲军队和需要驻扎的路径节点

    for (int i = head[1]; i; i = Next[i]) {
        if (need[ver[i]]) ned[++btot] = dist[ver[i]][0];
    }
    // 记录仍需要驻扎的节点的距离

    if (atot < btot) return false;

    sort(tim + 1, tim + atot + 1);
    sort(ned + 1, ned + btot + 1);

    int i = 1, j = 1;
    while (i <= btot && j <= atot) {
        if (tim[j] >= ned[i]) {
            i++, j++;
        } else {
            j++;
        }
    }
    // 贪心匹配剩余军队与未覆盖的路径节点

    return i > btot;
}
// 检查在当前时间限制下，是否能将所有路径覆盖

int main() {
    ll l = 0, r = 0, mid;
    cin >> n;
    t = log2(n) + 1;

    for (int i = 1; i < n; i++) {
        int x, y, z;
        scanf("%d%d%d", &x, &y, &z);
        add(x, y, z);
        r += z;
    }
    // 输入城市信息，并计算道路总长度r

    bfs();  // 调用BFS进行预处理

    cin >> m;
    for (int i = 1; i <= m; i++) {
        scanf("%d", &query[i]);
    }
    // 输入军队驻扎的城市

    while (l <= r) {
        mid = (l + r) >> 1;
        if (check(mid)) {
            r = mid - 1;
            ans = mid;
            ok = true;
        } else {
            l = mid + 1;
        }
    }
    // 通过二分搜索，找到最小的时间限制

    if (!ok) cout << -1;
    else cout << ans;
    // 输出最终结果

    return 0;
}
